@@ -83,16 +83,19 @@ job3.schedule().get_prev()
 
 import os
 import re
-import sys
+import shlex
 
 import codecs
+import logging
 import tempfile
 import subprocess as sp
 
-from datetime import date, datetime
+from time import sleep
+from collections import OrderedDict
+from datetime import time, date, datetime, timedelta
 
 __pkgname__ = 'python-crontab'
-__version__ = '1.9.5'
+__version__ = '2.0'
 
 ITEMREX = re.compile(r'^\s*([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)'
                      r'\s+([^@#\s]+)\s+([^#\n]*)(\s+#\s*([^\n]*)|$)')
@@ -116,11 +119,11 @@ SPECIALS = {"reboot":   '@reboot',
 SPECIAL_IGNORE = ['midnight', 'annually']
 
 S_INFO = [
-    {'name': 'Minutes',      'max': 59, 'min': 0},
-    {'name': 'Hours',        'max': 23, 'min': 0},
-    {'name': 'Day of Month', 'max': 31, 'min': 1},
-    {'name': 'Month',        'max': 12, 'min': 1, 'enum': MONTH_ENUM},
-    {'name': 'Day of Week',  'max': 6,  'min': 0, 'enum': WEEK_ENUM},
+    {'max': 59, 'min': 0, 'name': 'Minutes'},
+    {'max': 23, 'min': 0, 'name': 'Hours'},
+    {'max': 31, 'min': 1, 'name': 'Day of Month'},
+    {'max': 12, 'min': 1, 'name': 'Month', 'enum': MONTH_ENUM},
+    {'max': 6, 'min': 0, 'name': 'Day of Week', 'enum': WEEK_ENUM},
 ]
 
 # Detect Python3 and which OS for temperments.
@@ -129,10 +132,19 @@ PY3 = platform.python_version()[0] == '3'
 WINOS = platform.system() == 'Windows'
 SYSTEMV = not WINOS and os.uname()[0] in ["SunOS", "AIX", "HP-UX"]
 
-if not WINOS:
-    import pwd
+LOG = logging.getLogger('crontab')
 
 CRONCMD = "/usr/bin/crontab"
+SHELL = os.environ.get('SHELL', '/bin/sh')
+# The shell won't actually work on windows here, but
+# it should be updated later in the below conditional.
+
+current_user = lambda: None
+if not WINOS:
+    import pwd
+    def current_user():
+        """Returns the username of the current user"""
+        return pwd.getpwuid(os.getuid())[0]
 
 if PY3:
     # pylint: disable=W0622
@@ -140,22 +152,28 @@ if PY3:
     basestring = str
 
 
-def pipeOpen(cmd, *args, **flags):
+def open_pipe(cmd, *args, **flags):
     """Runs a program and orders the arguments for compatability.
 
     a. keyword args are flags and always appear /before/ arguments for bsd
     """
-    l = tuple(cmd.split(' '))
-    for (k,v) in flags.items():
-        if v is not None:
-            l += len(k)==1 and ("-%s" % (k,), str(v)) or ("--%s=%s" % (k,v),)
-    l += tuple(args)
-    return sp.Popen(tuple(a for a in l if a), stdout=sp.PIPE, stderr=sp.PIPE)
+    cmd_args = tuple(shlex.split(cmd))
+    for (key, value) in flags.items():
+        if len(key) == 1:
+            cmd_args += ("-%s" % key),
+            if value is not None:
+                cmd_args += str(value),
+        else:
+            cmd_args += ("--%s=%s" % (key, value)),
+    args = tuple(arg for arg in (cmd_args + tuple(args)) if arg)
+    return sp.Popen(args, stdout=sp.PIPE, stderr=sp.PIPE)
 
 def _unicode(text):
     """Convert to the best string format for this python version"""
     if type(text) is str and not PY3:
         return unicode(text, 'utf-8')
+    if type(text) is bytes and PY3:
+        return text.decode('utf-8')
     return text
 
 
@@ -178,6 +196,7 @@ class CronTab(object):
         self.lines = None
         self.crons = None
         self.filen = None
+        self.env = {}
         # Protect windows users
         self.root = not WINOS and os.getuid() == 0
         # Storing user flag / username
@@ -197,16 +216,19 @@ class CronTab(object):
 
     @property
     def user(self):
-        if self._user is True and pwd:
-            return pwd.getpwuid(os.getuid())[0]
+        """Return user's username of this crontab if applicable"""
+        if self._user is True:
+            return current_user()
         return self._user
 
     @property
     def user_opt(self):
+        """Returns the user option for the crontab commandline"""
         # Fedora and Mac require the current user to not specify
         # But Ubuntu/Debian doesn't care. Be careful here.
         if self._user and self._user is not True:
-            return {'u': self._user}
+            if self._user != current_user():
+                return {'u': self._user}
         return {}
 
     def read(self, filename=None):
@@ -216,6 +238,7 @@ class CronTab(object):
         """
         self.crons = []
         self.lines = []
+        self.env = OrderedDict()
         lines = []
         if self.intab is not None:
             lines = self.intab.split('\n')
@@ -224,24 +247,39 @@ class CronTab(object):
             with codecs.open(filename, 'r', encoding='utf-8') as fhl:
                 lines = fhl.readlines()
         elif self.user:
-            (out, err) = pipeOpen(CRONCMD, l='', **self.user_opt).communicate()
+            (out, err) = open_pipe(CRONCMD, l='', **self.user_opt).communicate()
             if err and 'no crontab for' in str(err):
                 pass
             elif err:
                 raise IOError("Read crontab %s: %s" % (self.user, err))
             lines = out.decode('utf-8').split("\n")
         for line in lines:
-            cron = CronItem(line, cron=self)
-            if cron.is_valid():
-                self.crons.append(cron)
-                self.lines.append(cron)
-            else:
-                self.lines.append(line.replace('\n', ''))
+            self.append(CronItem(line, cron=self), line, read=True)
 
-    def write(self, filename=None):
+    def append(self, cron, line='', read=False):
+        """Append a CronItem object to this CronTab"""
+        if cron.is_valid():
+            if read and not cron.comment and self.lines and \
+              self.lines[-1] and self.lines[-1][0] == '#':
+                cron.set_comment(self.lines.pop()[1:].strip())
+            self.crons.append(cron)
+            self.lines.append(cron)
+            return cron
+        if '=' in line:
+            if ' ' not in line or line.index('=') < line.index(' '):
+                (name, value) = line.split('=', 1)
+                self.env[name.strip()] = value.strip()
+                return None
+        self.lines.append(line.replace('\n', ''))
+
+    def write(self, filename=None, user=None):
         """Write the crontab to it's source or a given filename."""
         if filename:
             self.filen = filename
+        elif user is not None:
+            self.filen = None
+            self.intab = None
+            self._user = user
 
         # Add to either the crontab or the internal tab.
         if self.intab is not None:
@@ -261,23 +299,47 @@ class CronTab(object):
 
         if not self.filen:
             # Add the entire crontab back to the user crontab
-            pipeOpen(CRONCMD, path, **self.user_opt).wait()
-            os.unlink(path)
+            if self.user:
+                proc = open_pipe(CRONCMD, path, **self.user_opt)
+                # XXX This could do with being cleaned up quite a bit
+                proc.wait()
+                proc.stdout.close()
+                proc.stderr.close()
+                os.unlink(path)
+            else:
+                os.unlink(path)
+                raise IOError("Please specify user or filename to write.")
 
-    def write_to_user(self, user=None):
+    def write_to_user(self, user=True):
         """Write the crontab to a user (or root) instead of a file."""
-        self.filen = None
-        self.intab = None
-        if user is not None:
-            self._user = user
-        self.write()
+        return self.write(user=user)
+
+    def run_pending(self, **kwargs):
+        """Run all commands in this crontab if pending (generator)"""
+        for job in self:
+            ret = job.run_pending(**kwargs)
+            if ret is not None:
+                yield ret
+
+    def run_scheduler(self, timeout=0, **kwargs):
+        """Run the CronTab as an internal scheduler (generator)"""
+        count = 0
+        self.run_pending()
+        while count != timeout:
+            count += 1
+            sleep(kwargs.get('cadence', 60))
+            now = datetime.now()
+            if 'warp' in kwargs:
+                now += timedelta(seconds=count * 60)
+            for value in self.run_pending(now=now):
+                yield value
 
     def render(self):
         """Render this crontab as it would be in the crontab."""
-        crons = []
-        for cron in self.lines:
-            crons.append(unicode(cron))
-        result = u'\n'.join(crons)
+        envs = self.env.items()
+        env = ["%s=%s" % (key, _unicode(val)) for (key, val) in envs]
+        crons = [unicode(cron) for cron in self.lines]
+        result = u'\n'.join(env) + u'\n'.join(crons)
         if result and result[-1] not in (u'\n', u'\r'):
             result += u'\n'
         return result
@@ -290,10 +352,7 @@ class CronTab(object):
         """
         if not user and self.user is False:
             raise ValueError("User is required for system crontabs.")
-        item = CronItem(command=command, comment=comment, user=user, cron=self)
-        self.crons.append(item)
-        self.lines.append(item)
-        return item
+        return self.append(CronItem(None, command, comment, user, self))
 
     def find_command(self, command):
         """Return an iter of jobs matching any part of the command."""
@@ -331,15 +390,20 @@ class CronTab(object):
                 yield cron.comment
                 returned.append(cron.comment)
 
-    def remove_all(self, command=None, comment=None, time=None):
+    def remove_all(self, **kwargs):
         """Removes all crons using the stated command OR that have the
-        stated comment OR removes everything if no arguments specified."""
-        if command:
-            return self.remove(*self.find_command(command))
-        elif comment:
-            return self.remove(*self.find_comment(comment))
-        elif time:
-            return self.remove(*self.find_time(time))
+        stated comment OR removes everything if no arguments specified.
+
+           command - Remove all with this command
+           comment - Remove all with this comment or ID
+           time    - Remove all with this time code
+        """
+        if 'command' in kwargs:
+            return self.remove(*self.find_command(kwargs['command']))
+        elif 'comment' in kwargs:
+            return self.remove(*self.find_comment(kwargs['comment']))
+        elif 'time' in kwargs:
+            return self.remove(*self.find_time(kwargs['time']))
         return self.remove(*self.crons[:])
 
     def remove(self, *items):
@@ -358,6 +422,16 @@ class CronTab(object):
         self.lines.remove(item)
         return 1
 
+    def __repr__(self):
+        kind = 'System ' if self._user == False else ''
+        if self.filen:
+            return "<%sCronTab '%s'>" % (kind, self.filen)
+        elif self.user and not self.user_opt:
+            return "<My CronTab>"
+        elif self.user:
+            return "<User CronTab '%s'>" % self.user
+        return "<Unattached %sCronTab>" % kind
+
     def __iter__(self):
         return self.crons.__iter__()
 
@@ -380,13 +454,14 @@ class CronItem(object):
     May be considered to be a cron job object.
     """
     def __init__(self, line=None, command='', comment='', user=None, cron=None):
-        self.cron    = cron
-        self.user    = user
-        self.valid   = False
+        self.cron = cron
+        self.user = user
+        self.valid = False
         self.enabled = True
         self.special = False
         self.comment = None
         self.command = None
+        self.last_run = None
 
         self._log = None
 
@@ -404,7 +479,7 @@ class CronItem(object):
     def delete(self):
         """Delete this item and remove it from it's parent"""
         if not self.cron:
-            raise UnboundLocalError("Cron item is not associated with a crontab!")
+            raise UnboundLocalError("Cron item is not in a crontab!")
         else:
             self.cron.remove(self)
 
@@ -426,6 +501,7 @@ class CronItem(object):
         self._set_parse(SPECREX.findall(line))
 
     def _set_parse(self, result):
+        """Set all the parsed variables into the item"""
         if not result:
             return
         if self.cron.user == False:
@@ -464,7 +540,10 @@ class CronItem(object):
         result = u"%s %s%s" % (str(self.slices), user, self.command)
         if self.comment:
             self.comment = _unicode(self.comment)
-            result += u" # " + self.comment
+            if SYSTEMV:
+                result = "# " + self.comment + "\n" + result
+            else:
+                result += u" # " + self.comment
         if not self.enabled:
             result = u"# " + result
         return result
@@ -485,7 +564,7 @@ class CronItem(object):
         Many of these patterns exist as special tokens on Linux, such as
         `@midnight` and `@hourly`
         """
-        return ItemEveryInterface(self.slices, unit)
+        return Every(self.slices, unit)
 
     def setall(self, *args):
         """Replace existing time pattern with these five values given as args:
@@ -520,15 +599,37 @@ class CronItem(object):
         """Returns the number of times this item will execute in any hour"""
         return self.slices.frequency_per_hour()
 
+    def run_pending(self, now=None):
+        """Runs the command if scheduled"""
+        now = now or datetime.now()
+        if self.is_enabled():
+            if self.last_run is not None:
+                next_time = self.schedule(self.last_run).get_next()
+                if next_time < now:
+                    self.last_run = now
+                    return self.run()
+            else:
+                self.last_run = now
+
+    def run(self):
+        """Runs the given command as a pipe"""
+        shell = SHELL
+        if self.cron and 'SHELL' in self.cron.env:
+            shell = self.cron.env['SHELL']
+        (out, err) = open_pipe(shell, '-c', self.command).communicate()
+        if err:
+            LOG.error(err)
+        return out.strip()
+
     def schedule(self, date_from=None):
-        """Return a croniter schedule is available."""
+        """Return a croniter schedule if available."""
         if not date_from:
             date_from = datetime.now()
         try:
             # Croniter is an optional import
             from croniter.croniter import croniter
         except ImportError:
-            raise ImportError("Croniter is not available. Please install croniter"
+            raise ImportError("Croniter not available. Please install croniter"
                           " python module via pip or your package manager")
 
         class Croniter(croniter):
@@ -573,6 +674,7 @@ class CronItem(object):
 
     @property
     def day(self):
+        """Return the day slice"""
         return self.dom
 
     @property
@@ -601,8 +703,8 @@ class CronItem(object):
     def __len__(self):
         return len(str(self))
 
-    def __getitem__(self, x):
-        return self.slices[x]
+    def __getitem__(self, key):
+        return self.slices[key]
 
     def __lt__(self, value):
         return self.frequency() < CronSlices(value).frequency()
@@ -615,11 +717,12 @@ class CronItem(object):
 
     def __unicode__(self):
         if not self.is_valid():
-            raise ValueError('Refusing to render invalid crontab. Disable to continue.')
+            raise ValueError('Refusing to render invalid crontab.'
+                             ' Disable to continue.')
         return self.render()
 
 
-class ItemEveryInterface(object):
+class Every(object):
     """Provide an interface to the job.every() method:
         Available Calls:
           minute, minutes, hour, hours, dom, doms, month, months, dow, dows
@@ -630,18 +733,19 @@ class ItemEveryInterface(object):
     def __init__(self, item, units):
         self.slices = item
         self.unit = units
-        for (x, i) in enumerate(['minute', 'hour', 'dom', 'month', 'dow',
+        for (key, name) in enumerate(['minute', 'hour', 'dom', 'month', 'dow',
                                  'min', 'hour', 'day', 'moon', 'weekday']):
-            setattr(self, i, self._set(x % 5))
-            setattr(self, i+'s', self._set(x % 5))
+            setattr(self, name, self.set_attr(key % 5))
+            setattr(self, name+'s', self.set_attr(key % 5))
 
-    def _set(self, target):
+    def set_attr(self, target):
+        """Inner set target, returns function"""
         def innercall():
             """Returned inner call for setting slice targets"""
             self.slices.clear()
             # Day-of-week is actually a level 2 set, not level 4.
-            for p in range(target == 4 and 2 or target):
-                self.slices[p].on('<')
+            for key in range(target == 4 and 2 or target):
+                self.slices[key].on('<')
             self.slices[target].every(self.unit)
         return innercall
 
@@ -651,13 +755,13 @@ class ItemEveryInterface(object):
             raise ValueError("Invalid value '%s', outside 1 year" % self.unit)
         self.slices.setall('@yearly')
 
-
 class CronSlices(list):
     """Controls a list of five time 'slices' which reprisent:
         minute frequency, hour frequency, day of month frequency,
         month requency and finally day of the week frequency.
      """
     def __init__(self, *args):
+        super(CronSlices, self).__init__()
         for info in S_INFO:
             self.append(CronSlice(info))
         self.special = None
@@ -667,50 +771,59 @@ class CronSlices(list):
 
     def is_self_valid(self, *args):
         """Object version of is_valid"""
-        return CronSlices.is_valid(*(args or [self]))
+        args = args or (self,)
+        return CronSlices.is_valid(*args)
 
     @classmethod
     def is_valid(cls, *args):
         """Returns true if the arguments are valid cron pattern"""
         try:
             return bool(cls(*args))
-        except:
+        except Exception:
             return False
 
-    def setall(self, to, *slices):
+    def setall(self, value, *slices):
         """Parses the various ways date/time frequency can be specified"""
         self.clear()
-        if isinstance(to, CronItem):
-            slices = to.slices
-        elif isinstance(to, list):
-            slices = to
+        if isinstance(value, CronItem):
+            slices = value.slices
+        elif isinstance(value, datetime):
+            slices = [value.minute, value.hour, value.day, value.month, '*']
+        elif isinstance(value, time):
+            slices = [value.minute, value.hour, '*', '*', '*']
+        elif isinstance(value, date):
+            slices = [0, 0, value.day, value.month, '*']
+          # It might be possible to later understand timedelta objects
+          # but there's no convincing mathematics to do the conversion yet.
+        elif isinstance(value, list):
+            slices = value
         elif slices:
-            slices = (to,) + slices
-        elif isinstance(to, basestring) and to:
-            if to.count(' ') == 4:
-                slices = to.strip().split(' ')
-            elif to.strip()[0] == '@':
-                to = to[1:]
+            slices = (value,) + slices
+        elif isinstance(value, basestring) and value:
+            if value.count(' ') == 4:
+                slices = value.strip().split(' ')
+            elif value.strip()[0] == '@':
+                value = value[1:]
                 slices = None
             else:
-                slices = [to]
+                slices = [value]
 
-            if 'reboot' in (to, to[1:]):
+            if 'reboot' in (value, value[1:]):
                 self.special = '@reboot'
                 return True
-            elif to in SPECIALS.keys():
-                return self.setall(SPECIALS[to])
+            elif value in SPECIALS.keys():
+                return self.setall(SPECIALS[value])
             elif not slices:
                 return False
 
         if id(slices) == id(self):
             raise AssertionError("Can not set cron to itself!")
 
-        for a, b in zip(self, slices):
+        for set_a, set_b in zip(self, slices):
             try:
-                a.parse(b)
+                set_a.parse(set_b)
             except ValueError as error:
-                sys.stderr.write("WARNING: %s\n" % str(error))
+                LOG.warning(str(error))
                 return False
             except Exception:
                 return False
@@ -722,22 +835,23 @@ class CronSlices(list):
 
     def render(self):
         "Return just the first part of a cron job (the numbers or special)"
-        time = self.clean_render()
+        slices = self.clean_render()
         if self.special:
             return self.special
         elif not SYSTEMV:
             for (name, value) in SPECIALS.items():
-                if value == time and name not in SPECIAL_IGNORE:
+                if value == slices and name not in SPECIAL_IGNORE:
                     return "@%s" % name
-        return time
+        return slices
 
     def clear(self):
         """Clear the special and set values"""
         self.special = None
-        for s in self:
-            s.clear()
+        for item in self:
+            item.clear()
 
     def frequency(self, year=None):
+        """Return frequence per year times frequency per day"""
         return self.frequency_per_year(year=year) * self.frequency_per_day()
 
     def frequency_per_year(self, year=None):
@@ -774,8 +888,25 @@ class CronSlices(list):
 
 
 class SundayError(KeyError):
+    """Sunday was specified as 7 instead of 0"""
     pass
 
+class Also(object):
+    """Link range values together (appending instead of replacing)"""
+    def __init__(self, obj):
+        self.obj = obj
+
+    def every(self, *a):
+        """Also every one of these"""
+        return self.obj.every(*a, also=True)
+
+    def on(self, *a):
+        """Also on these"""
+        return self.obj.on(*a, also=True)
+
+    def during(self, *a):
+        """Also during these"""
+        return self.obj.during(*a, also=True)
 
 class CronSlice(object):
     """Cron slice object which shows a time pattern"""
@@ -798,7 +929,7 @@ class CronSlice(object):
                 self.parts.append(self.get_range(part))
             else:
                 try:
-                    self.parts.append(self._v(part))
+                    self.parts.append(self.parse_value(part))
                 except SundayError:
                     self.parts.append(0)
                 except ValueError as err:
@@ -837,9 +968,9 @@ class CronSlice(object):
         """Set the time values to the specified placements."""
         if not opts.get('also', False):
             self.clear()
-        for av in n_value:
+        for set_a in n_value:
             try:
-                self.parts += self._v(av),
+                self.parts += self.parse_value(set_a),
             except SundayError:
                 self.parts += 0,
         return self.parts
@@ -848,28 +979,13 @@ class CronSlice(object):
         """Set the During value, which sets a range"""
         if not also:
             self.clear()
-        self.parts.append(self.get_range(self._v(vfrom), self._v(vto)))
+        self.parts.append(self.get_range(self.parse_value(vfrom), self.parse_value(vto)))
         return self.parts[-1]
 
     @property
     def also(self):
         """Appends rather than replaces the new values"""
-        outself = self
-
-        class Also(object):
-            """Will append new values"""
-            def every(self, *a):
-                """Also every one of these"""
-                return outself.every(*a, also=True)
-
-            def on(self, *a):
-                """Also on these"""
-                return outself.on(*a, also=True)
-
-            def during(self, *a):
-                """Also during these"""
-                return outself.during(*a, also=True)
-        return Also()
+        return Also(self)
 
     def clear(self):
         """clear the slice ready for new vaues"""
@@ -881,46 +997,47 @@ class CronSlice(object):
 
     def __iter__(self):
         """Return the entire element as an iterable"""
-        r = {}
+        ret = {}
         # An empty part means '*' which is every(1)
         if not self.parts:
             self.every(1)
         for part in self.parts:
             if isinstance(part, CronRange):
                 for bit in part.range():
-                    r[bit] = 1
+                    ret[bit] = 1
             else:
-                r[int(part)] = 1
-        for x in r:
-            yield x
+                ret[int(part)] = 1
+        for val in ret:
+            yield val
 
     def __len__(self):
         """Returns the number of times this slice happens in it's range"""
         return len(list(self.__iter__()))
 
-    def _v(self, v):
-        if v == '>':
-            v = self.max
-        elif v == '<':
-            v = self.min
+    def parse_value(self, val):
+        """Parse the value of the cron slice and raise any errors needed"""
+        if val == '>':
+            val = self.max
+        elif val == '<':
+            val = self.min
         try:
-            out = get_cronvalue(v, self.enum)
+            out = get_cronvalue(val, self.enum)
         except ValueError:
-            raise ValueError("Unrecognised '%s'='%s'" % (self.name, v))
+            raise ValueError("Unrecognised '%s'='%s'" % (self.name, val))
         except KeyError:
-            raise KeyError("No enumeration '%s' got '%s'" % (self.name, v))
+            raise KeyError("No enumeration '%s' got '%s'" % (self.name, val))
 
         if self.max == 6 and int(out) == 7:
             raise SundayError("Detected Sunday as 7 instead of 0!")
 
         if int(out) < self.min or int(out) > self.max:
             raise ValueError("Invalid value '%s', expected %d-%d for %s" % (
-                str(v), self.min, self.max, self.name))
+                str(val), self.min, self.max, self.name))
         return out
 
-    def filter_v(self, v):
+    def filter_value(self, val):
         """Support wrapper for enumerations and check for range"""
-        return self._v(v)
+        return self.parse_value(val)
 
 
 def get_cronvalue(value, enums):
@@ -991,14 +1108,14 @@ class CronRange(object):
         """Parse a ranged value in a cronjob"""
         if value.count('/') == 1:
             value, seq = value.split('/')
-            self.seq = self.slice.filter_v(seq)
+            self.seq = self.slice.filter_value(seq)
             if self.seq < 1 or self.seq > self.slice.max - 1:
                 raise ValueError("Sequence can not be divided by zero or max")
         if value.count('-') == 1:
             vfrom, vto = value.split('-')
-            self.vfrom = self.slice.filter_v(vfrom)
+            self.vfrom = self.slice.filter_value(vfrom)
             try:
-                self.vto = self.slice.filter_v(vto)
+                self.vto = self.slice.filter_value(vto)
             except SundayError:
                 self.vto = 6
         elif value == '*':
@@ -1019,7 +1136,7 @@ class CronRange(object):
         if self.seq != 1:
             value += "/%d" % self.seq
         if value != '*' and SYSTEMV:
-            value = ','.join(map(str, self.range()))
+            value = ','.join([str(val) for val in self.range()])
         return value
 
     def range(self):
