@@ -31,7 +31,11 @@ try:
 except ImportError:
     from test import support as test_support
 
+TEST_FILE = os.path.join(os.path.dirname(__file__), 'data', 'crontab')
 INITAL_TAB = """
+VAR=foo
+JAR=bar
+
 */30 * * * * palin one_cross_each
 """
 
@@ -40,6 +44,10 @@ class SystemCronTestCase(unittest.TestCase):
     def setUp(self):
         self.crontab = CronTab(tab=INITAL_TAB, user=False)
 
+    def test_00_repr(self):
+        """System crontab repr"""
+        self.assertEqual(repr(self.crontab), "<Unattached System CronTab>")
+
     def test_01_read(self):
         """Read existing command"""
         jobs = 0
@@ -54,7 +62,9 @@ class SystemCronTestCase(unittest.TestCase):
         job = self.crontab.new(command='release_brian', user='pontus')
         self.assertEqual(job.user, 'pontus')
         self.assertEqual(job.command, 'release_brian')
-        self.assertEqual(str(self.crontab), """
+        self.assertEqual(str(self.crontab), """VAR=foo
+JAR=bar
+
 */30 * * * * palin one_cross_each
 
 * * * * * pontus release_brian
@@ -72,7 +82,9 @@ class SystemCronTestCase(unittest.TestCase):
     def test_04_remove(self):
         """Remove the user flag"""
         self.crontab._user = None
-        self.assertEqual(str(self.crontab), """
+        self.assertEqual(str(self.crontab), """VAR=foo
+JAR=bar
+
 */30 * * * * one_cross_each
 """)
         self.crontab.new(command='now_go_away')
@@ -91,6 +103,19 @@ class SystemCronTestCase(unittest.TestCase):
         crontab = CronTab(user=False, tab="* * * * * user command\n")
         self.assertEqual(str(crontab), "* * * * * user command\n")
 
+    def test_07_variables(self):
+        """Vixie cron variables support"""
+        self.assertEqual(self.crontab.env, {'VAR': 'foo', 'JAR': 'bar'})
+        self.crontab.env['SHELL'] = 'bash'
+        self.assertEqual(str(self.crontab), """VAR=foo\nJAR=bar\nSHELL=bash\n
+*/30 * * * * palin one_cross_each
+""")
+
+    def test_08_system_file(self):
+        """Load system crontab from a file"""
+        crontab = CronTab(user=False, tabfile=TEST_FILE)
+        self.assertEqual(repr(crontab), "<System CronTab '%s'>" % TEST_FILE)
+
 if __name__ == '__main__':
     test_support.run_unittest(
        SystemCronTestCase,
